/*
 * @class controller.BizDomainPromptController Creates a BizDomainPromptModel and passes a list of analytic domains to it
 */
define([
    'sna/model/ApplicationContextModel',
    'sna/util/I18nUtil',
    '../util/ContextPreferenceUtil',
    '../util/AnalyticDomainUtil',
    'sna/util/OverlayUtil',
    'dojo/dom-style',
    'dijit/registry',
    'sna/model/BizDomainPromptModel',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/dom',
    'sna/data/BizDomainPromptData',
    'dojo/data/ObjectStore',
    'dojo/store/Memory'
], function(ApplicationContextModel, I18nUtil, ContextPreferenceUtil, AnalyticDomainUtil, OverlayUtil, domStyle, registry, Model, declare, lang, dom, bizDomainPromptData, ObjectStore, Memory ) {
    return declare([], {
        constructor: function( args ) {
            this.model = new Model();
	        this.model.controller = this;
	        
	        this.advancedSearchController = args.advancedSearchController;
	        this.alertsViewController = args.alertsViewController;
            this.alertDetailsController = args.alertDetailsController;
	        this.viewSwitcherController = args.viewSwitcherController;
            this.globalToolbarController = args.globalToolbarController;
            this.sessionMonitorController = args.sessionMonitorController;
	        
	        this.data = bizDomainPromptData;

            this.inherited(arguments);
        },

        /**
         * starts retrieval of analytic domains and then passes the results to the model 
         */
        start: function( hideCloseButton ) {
            var that = this;

//            sessionTimeoutMonitor = new SessionTimeoutMonitor({});
//            this._startSessionTimeout();

            bizDomainPromptData.getVerticals(function(data) {
                /*if(data.error){
                    new LogoffDialog({
                        closable: false,
                        message: data.error.code
                    });
                } else {*/
                that.globalToolbarController.updateAnalyticDomainsList(bizDomainPromptData.getVerticalsStore(data));
                that.model.render(data, hideCloseButton);
//                }
            });
        },
		
        /**
         * Starts advanced search or the alerts view after a domain has been selected and the 'Ok' button clicked
         * 
         * @param vertical selected analytic domain
         */
        bizDomainSelect: function( vertical ) {
            // display the unsaved changes dialog if the context is being switched, save is enabled, and the details view is loaded
           if( ApplicationContextModel.get('unsavedChanges') && this.alertDetailsController.isLoaded()) {
               var confirm = window.confirm(I18nUtil.get('global_unsavedchanges'));
               if( confirm === true ) {
                   ApplicationContextModel.set('unsavedChanges', false);
               }
               else {
                   return;
               }
           }

            OverlayUtil.showOverlay();

            var that = this;
            this.advancedSearchController.loadAndStartAdvancedSearch(vertical, true, function(hasAdvancedSearch) {
                that.sessionMonitorController.start();

                if( hasAdvancedSearch === false ) {
                    // note: according to the admin guide, if there is no advanced search dialog a single
                    // query parameter with the following properties is submitted.
                    var submissions = {
                        queryParams: [{
                            queryType: 'A',
                            queryParameter: 'A',
                            queryValue: 'a'
                        }]
                    };

                    that.viewSwitcherController.switchToAlertsListPage(true, vertical, submissions);
                }

                // display the vertical name
                AnalyticDomainUtil.setAnalyticDomain(lang.replace(vertical));
                that.globalToolbarController.displayCurrentAnalyticDomain(vertical);
            });
        }

        /*_startSessionTimeout: function() {
            this.sessionMonitorController.start();
        }*/
    });
});

