/**
 * @class controller.ViewSwitcherController Central place for all view switching logic
 * 
 *  Handles invoking and cleaning
 */
define([
    'sna/model/ApplicationContextModel',
    'sna/util/I18nUtil',
    'dojo/query',
    'dojo/_base/array',
    'dijit/registry',
    'dojo/_base/lang',
    'dojo/_base/declare'
], function(ApplicationContextModel, I18nUtil, query, array, registry, lang, declare ) {
	var cleanupViews = function() {
		this.alertDetailsController.cleanup();
		this.alertsViewController.cleanup();
    };
	
	return declare([], {
		constructor: function( args ) {
		    this.pageToolbarController = args.pageToolbarController;
		    this.alertsViewController = args.alertsViewController;
		    this.alertDetailsController = args.alertDetailsController;
		},
		
		/**
		 * this function should be the only way that anyone switches to the alerts view
		 * @param startAlerts if true, will call the {@link controller.AlertsViewController} start function
		 * @param vertical the analytic domain
		 * @param submissions advanced search parameters
		 * @param fetchEntities if true, the alerts view will fetch the alerts from the server. Else, none will be fetched
         * @param useCached if true, will use cached data from the alerts list instead of doing another server call
		 */
		switchToAlertsListPage: function( startAlerts, vertical, submissions, fetchEntities, useCached ) {
            if( ApplicationContextModel.get('unsavedChanges') ) {
                var confirm = window.confirm(I18nUtil.get('global_unsavedchanges'));
                if( confirm === true ) {
                    ApplicationContextModel.set('unsavedChanges', false);
                }
                else {
                    return;
                }
            }

			if(!submissions) {
				console.warn('ViewSwitcherController: advanced search parameters were undefined', submissions);
			}
			
			lang.hitch(this, cleanupViews)();
			
			this.pageToolbarController.enterAlertsList(startAlerts, vertical, submissions, useCached);
			this.advancedSearchParameters = submissions; 
			
			if(startAlerts) {
			    this.alertsViewController.start(vertical, submissions, fetchEntities, useCached);
            }
		},

        /**
         *
         * @param entity
         * @param vertical
         * @param submissions advanced search parameters
         */
		switchToDetailsPage: function( entity, vertical, submissions ) {
			lang.hitch(this, cleanupViews)();
			
			this.pageToolbarController.enterAlertDetails(vertical, submissions);
			this.alertDetailsController.start(entity, vertical);

		},
		
		switchToNetworkView: function() {
			
		},
		
		/**
		 * added this function because {@link controller.RefreshController} needs access to advanced search parameters and it has no other way to get them.
		 * TODO perhaps think of a more elegant way to design this? 
		 */
		getAdvancedSearchParameters: function() {
			return this.advancedSearchParameters;
		}
	});
});