/**
 * @description 
 * 
 * @class
 */
define([
    '../../util/UserPreferenceUtil',
    'dijit/DropDownMenu',
    'dijit/form/Button',
    'dijit/form/CheckBox',
    'dijit/form/DateTextBox',
    'dijit/form/DropDownButton',
    'dijit/form/HorizontalSlider',
    'dijit/form/TextBox',
    'dijit/layout/ContentPane',
    'dijit/MenuItem',
    'dijit/registry',
    'dijit/TitlePane',
    'dijit/Toolbar',
    'dijit/Tooltip',
    'dojo/_base/array',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/aspect',
    'dojo/dom',
    'dojo/dom-construct',
    'dojo/dom-style',
    'dojo/mouse',
    'dojo/on',
    'dojo/ready',
    'dojox/form/RangeSlider',
    'dojox/lang/functional',
    'sna/model/widgets/PreferencesCheckboxGroup',
    'sna/model/widgets/PreferencesDate',
    'sna/model/widgets/PreferencesDateTextBox',
    'sna/model/widgets/PreferencesDateTimeGroup',
    'sna/model/widgets/PreferencesHorizontalSlider',
    'sna/model/widgets/PreferencesTextBox',
    'sna/util/I18nUtil'
], function(UserPreferenceUtil, DropDownMenu, Button, Checkbox, DateTextBox, DropDownButton, HorizontalSlider, TextBox, ContentPane, MenuItem, registry, TitlePane, Toolbar, Tooltip, array, declare, lang,
		     aspect, dom, domConstruct, domStyle, mouse, on, ready, RangeSlider, functional, PreferencesCheckboxGroup, PreferencesDate, PreferencesDateTextBox,
		     PreferencesDateTimeGroup, PreferencesHorizontalSlider, PreferencesTextBox, i18nUtil ) {
	
	// inner class with functions to prepare facets
	var FacetPreparer = {

	    // generate text facet from json object
	    prepareTextFacet: function( facet, onChangeCallback, mode ) {
			var textbox = new PreferencesTextBox({
				name: facet.key,
				formattedKey: facet.formattedKey,
				value: '',
				onKeyUp: onChangeCallback,
				onChange: onChangeCallback,
                // dont' know how to access onKeyUp directly inside PreferencesTextBox, so pass it in
                keyUpCallback: onChangeCallback,
                useSize: false, // since facets cannot be of a pre-defined size, make the textbox just auto-size
                labelPlacement: 'left',
                useTitlePane: true,
                innerLabel: facet.formattedKey + ':',
                enabled: facet.valid,
                errorCode: facet.errorCode,
                placeHolder: facet.placeHolder || '', 
				
				snamode: mode
			});
			
			return textbox;
		},
		
		// generate checklist facet from json object
		prepareChecklistFacet: function( facet, onChangeCallback, mode ) {
			var preferencesCheckboxGroup = new PreferencesCheckboxGroup({
				key: facet.key,
				formattedKey: facet.formattedKey,
                maxColumns: 1, // in facets, it's one checkbox per line,
                useTitlePane: true,
                enabled: facet.valid,
                errorCode: facet.errorCode, 
				
				snamode: mode
			});
			
			// create individual Checkbox objects from each string label and add them to group
			functional.forIn( facet.values, function( value, index ) {
				preferencesCheckboxGroup.addCheckbox({
					name: value,
					value: value,
					formattedKey: value,
					onChangeCallback: onChangeCallback
				});
			});

			return preferencesCheckboxGroup;
		},
		
		// generate horizontal slider facet from json object
		prepareRangeFacet: function( facet, onChangeCallback, mode ) {
			var slider = new PreferencesHorizontalSlider({
				name: facet.key,
				key: facet.key,
				formattedKey: facet.formattedKey,
				minimum: facet.min,
				maximum: facet.max,
				value: [facet.min,facet.max],
				style: 'width:100%;',
                useTitlePane: true,
                enabled: facet.valid,
                errorCode: facet.errorCode,
				onChangeCallback: onChangeCallback,
				
				snamode: mode
			});
			
			return slider;
		},

        // generate daterange facet from json object
        prepareDateTimeRangeFacet: function( facet, onChangeCallback, mode ) {
            var preferencesDateTextBox = new PreferencesDateTimeGroup({
                key: facet.key,
                name: facet.key,
                formattedKey: facet.formattedKey,
                useTitlePane: true,
                enabled: facet.valid,
                errorCode: facet.errorCode,
                dateTextBoxFrom: new PreferencesDate({
                    name: facet.key,
                    value: undefined,
                    constraints: {
                        //min: '2003-01-01'
                        formatLength: 'short'
                    },
                    onChange: function() {
                        this._cachedValue = undefined;
                        onChangeCallback();
                    }
                }),

                dateTextBoxTo: new PreferencesDate({
                    name: facet.key,
                    value: undefined,
                    constraints: {
                        //min: '2003-01-01'
                        formatLength: 'short'
                    },
                    onChange: function() {
                        this._cachedValue = undefined;
                        onChangeCallback();
                    }
                }), 
				
                snamode: mode
            });

            return preferencesDateTextBox;
        },

		// generate daterange facet from json object
		prepareDateRangeFacet: function( facet, onChangeCallback, mode ) {
			var preferencesDateTextBox = PreferencesDateTextBox({
				key: facet.key,
				name: facet.key,
				formattedKey: facet.formattedKey,
                useTitlePane: true,
                enabled: facet.valid,
                errorCode: facet.errorCode,
                dateTextBoxFrom: PreferencesDate({
                    name: facet.key,
                    value: undefined,
                    constraints: {
                        //min: '2003-01-01'
                        formatLength: 'short'
                    },
                    onChange: function() {
                        this._cachedValue = undefined;
                        onChangeCallback();
                    }
                }),
				dateTextBoxTo: new PreferencesDate({
					name: facet.key,
					value: undefined,
					constraints: {
						//min: '2003-01-01'
						formatLength: 'short'
					},
                    onChange: function() {
                        this._cachedValue = undefined;
                        onChangeCallback();
                    }
				}), 
				
				snamode: mode
			});
			
			return preferencesDateTextBox;
		}
	}; // end of FacetPreparer
	
	// clear the values of all the passed-in facets
	var clearFacetPreferenceValues = function( facets, callback ) {	
		functional.forIn(facets, function( facet, index ) {
			facet.clear();
		});
		
		if( callback ) {
			callback();
		}
	};
	
	// set the values of the passed-in facets with the passed-in data
	var setFacetPreferenceValues = function( facets, data ) {
		functional.forIn(data, function( o, index ) {
			if( facets[index].setField ) {
				facets[index].setField('value', o.value);
			}
		});
	};
	
	// return a map of facetindex -> preference value
	var enumerateFacets = function( facets ) {
		var enumeration = {};
		
		functional.forIn(facets, function( facet, index ) {
			enumeration[index] = facet.getPreferenceValue();
		});
		
		return enumeration;
	};
	
	//
	var _prepareFacet = function( facet, facetEventMap, mode ) {
    	var type = facet.type;

        // unescape facet ids because if they contain non-english characters they will come back escaped in 3.1
        var unescaped = JSON.parse('"%0"'.tokenize(facet.key));
        var element = domConstruct.toDom('<span>%0</span>'.tokenize(unescaped));
        facet.key = element.innerText || element.textContent;

        if (type === 'text') {
            var newFacet = FacetPreparer.prepareTextFacet(facet, facetEventMap.onFacetChange, mode);
        }
        else if (type === 'checklist') {
        	var newFacet = FacetPreparer.prepareChecklistFacet(facet, facetEventMap.onFacetChange, mode);
        }
        else if (type === 'range') {
        	var newFacet = FacetPreparer.prepareRangeFacet(facet, facetEventMap.onFacetChange, mode);
        }
        else if (type === 'daterange') {
        	var newFacet = FacetPreparer.prepareDateRangeFacet(facet, facetEventMap.onFacetChange, mode);
        }
        else if (type === 'datetimerange') {
            var newFacet = FacetPreparer.prepareDateTimeRangeFacet(facet, facetEventMap.onFacetChange, mode);
        }

        // optional overriding filter + render functions
        newFacet.customFilter = facet.customFilter;
        newFacet.customRender = facet.customRender;

        return newFacet;
    };

	
	/**
	 * Start defining FacetHelper
	 */
	var helper = declare(null, {
    });
	
	// just in case a facetEventMap isn't given, 
	//   this can be used to avoid exceptions during callbacks
	helper.defaultFacetEventMap = {
	    onFacetChange: function() {
	    }
	};

	/**
	 * Add a toolbar to the passed-in container with the typical preference buttons (saving, retrieving, etc...)
	 */
	helper.addPreferenceButtonsToolbar = function( toolbarContainer, facets, userPreferenceKey, facetEventMap ) {
		// the toolbar that will hold the preference buttons
        var toolbar = new Toolbar({
        });

        // add apply filters button
        toolbar.addChild(new Button({
        	label: i18nUtil.get('alert_list_facet_apply'),
            showLabel:false,
            iconClass:'sasFilterIcon',
            onClick:function () {
        	    UserPreferenceUtil.retrieve(userPreferenceKey, function (data) {
        	    	setFacetPreferenceValues(facets, data);
                });
            }
        }));
        
        // add clear search button
        toolbar.addChild(new Button({
        	label: i18nUtil.get('alert_list_facet_clear'),
            showLabel:false,
            iconClass:'sasClearAllIcon',
            onClick:function () {
                clearFacetPreferenceValues(facets, facetEventMap.onFacetChange);
            }
        }));
        
        // add save current filters button
        toolbar.addChild(new Button({
        	label: i18nUtil.get('alert_list_facet_save'),
            showLabel:false,
            iconClass:'sasSaveIcon',
            onClick:function () {
        	    UserPreferenceUtil.save(userPreferenceKey, enumerateFacets(facets));
            }
        }));
        
        // add restore filters button
        toolbar.addChild(new Button({
            label: i18nUtil.get('alert_list_facet_restore'),
            showLabel:false,
            iconClass:'sasResetIcon',
            onClick:function () {
                clearFacetPreferenceValues(facets, facetEventMap.onFacetChange);
                UserPreferenceUtil.remove(userPreferenceKey);
            }
        }));

        // add toolbar to container
        toolbarContainer.addChild(toolbar);
        
        return toolbar;
    };
    
    /**
     * return a map of Facet component objects, that represent the passed-in facet json data
     */
    helper.prepareFacets = function( facetJsonMap, facetEventMap, mode ) {
        var facets = {};

        if (!facetEventMap) {
            facetEventMap = helper.defaultFacetEventMap;
        }

        functional.forIn(facetJsonMap, function (facetJsonObject, index) {
        	facets[facetJsonObject.key] = _prepareFacet(facetJsonObject, facetEventMap, mode);
        });
        
        return facets;
    };
    
    /**
     * 
     */
    helper.prepareFacet = function( facetJsonObject, facetEventMap, mode ) {
    	if( !facetEventMap ) {
    		facetEventMap = helper.defaultFacetEventMap;
    	};
    	
    	return _prepareFacet(facetJsonObject, facetEventMap, mode);
    };
    
    /**
     * Have each passed-in facet render within the passed-in container
     */
    helper.renderFacets = function( facets, container ) {
    	functional.forIn(facets, function( facet, index ) {
           helper.renderFacet(facet, container);  
    	});
    };

    helper.renderFacet = function(facet, container) {
         if(facet.customRender) {
            facet.customRender(container, facet);
         }
         else {
            facet.render(container);
         }
    };
    
    /**
     */
    helper.destroyFacets = function( facets ) {
    	functional.forIn(facets, function( facet, index ) {
    		facet.destroyFacet();
    	});
    };

    /**
     * renders the global search facet to another container
     * @param globalSearch the container to render to
     * @param d_container the original container
     * @param facet the facet object
     */
    helper.globalSearchRenderer = function(globalSearch, d_container, facet) {
        if(!facet.hide) {
            facet.hide = function() {
                domStyle.set(facet.domNode, 'display', 'none');
            };
            facet.show = function() {
                domStyle.set(facet.domNode, 'display', '');
            };
        }

        var globalSearchText = facet.fullPlaceHolder || facet.placeHolder;
        facet.fullPlaceHolder = globalSearchText;

        // truncate due to i18n concerns. 18 is arbitrary but fits in the global search textbox
        if(globalSearchText.length > 18) {
            globalSearchText = globalSearchText.substr(0, 17) + '...';
            facet.set('placeHolder', globalSearchText);

            // create a tooltip when facet placeholder text gets too long
            on(facet, mouse.enter, function(){
                Tooltip.show(facet.fullPlaceHolder, facet.domNode);
            });

            on(facet, mouse.leave, function(){
                Tooltip.hide(facet.domNode);
            });
        }

        facet.render(d_container);
        // move this node to global search
        domConstruct.place(facet.domNode, globalSearch);
        // destroy the original container
        facet.d_pane.destroyRecursive();
    };

    /**
     * creates the global search facet data object (not the final facet object)
     */
    helper.addGlobalSearchFacet = function(facetDataMap) {
        facetDataMap['global_search_facet'] = {
            type: 'text',
            key: 'global_search_facet',
            placeHolder: i18nUtil.get('global_search')
        };

        return facetDataMap['global_search_facet'];
    };
    
    return helper;
});
