/**
 * @class
 * Wrapper around the DateTextBox dijit component
 * 
 * Provides functions that allow User Preferences to be set
 */
define([
    './PreferencesDefaultRendererMixin',
    'dojo/dom-construct',
    'dijit/_WidgetBase',
    'dojo/_base/declare',
    'dijit/layout/ContentPane',
    'dijit/form/DateTextBox'
], function(PreferencesDefaultRendererMixin,  domConstruct, _WidgetBase, declare, ContentPane, DateTextBox) {
    return declare([DateTextBox, PreferencesDefaultRendererMixin], {
        constructor: function(args) {
            this.id = args.id;
            this.key = args.key;
            this.formattedKey = args.formattedKey;
            this.name = args.name;
            this.declaredClass = 'sas.sna.model.widgets.DateTextBox';
            this.hasDownArrow = false;
            this.inherited(arguments);
        },

        /**
         * set a field on this component
         */
        setField: function( field, ob ) {
            this.set(field, ob);
        },
        
        /**
         * get a field on this component
         */
        getField: function( field ) {
            return this[field];
        },
        
        buildRendering: function() {
        	this.inherited(arguments);
        	
        	var calendarIcon = domConstruct.toDom('<div class="sasDateIcon"></div>');
        	domConstruct.place(calendarIcon, this.domNode, 'first');
        },

        getCachedValue: function() {
            if(typeof(this._cachedValue) !== 'undefined') {
                if(this._cachedValue !== null) {
                    return new Date(this._cachedValue.getTime());
                }
                return this._cachedValue;
            }
            else {
                this._cachedValue = this.get('value');

                return this._cachedValue;
            }
        },

        /**
		 * Return an object representing the state of this component to save in User Preferences 
		 */
        getPreferenceValue:function () {
            return {
                'value':this.get('value')
            };
        },

        /**
		 * Clear the value of this component 
		 */
        clear:function () {
            this.set('value', null);
        }
    });
});
