/**
 * @class 
 * Wrapper around the TextBox diijt component
 *
 * Provides functions that allow User Preferences to be set
 */
define([
    'dojo/on',
    'dojo/dom-class',
    './PreferencesDefaultRendererMixin',
    'dojo/dom-style',
    'dojo/dom-attr',
    'dojo/dom-construct',
    'dijit/form/TextBox',
    'dijit/_WidgetBase',
    'dojo/_base/declare',
    'dijit/layout/ContentPane'
], function(on, domClass, PreferencesDefaultRendererMixin, domStyle, domAttr, domConstruct, TextBox, _WidgetBase, declare, ContentPane ) {
    return declare([TextBox, PreferencesDefaultRendererMixin], {
        intermediateChanges: true,

        constructor: function(args) {
            this.id = args.id;
            this.name = args.key;
            this.key = args.key;
            this.formattedKey = args.formattedKey;
            this.value = args.text;
            this.keyUpCallback = args.keyUpCallback;
            this.maxLength = args.maxLength;
            this.size = args.size || 25;
            this.useSize = args.useSize;
            if(typeof(this.useSize) === 'undefined') {
                this.useSize = true;
            }
            this.declaredClass = 'dijit.form.TextBox'; // because we're mixing in multiple classes
            args.tableWidth = '100%';
            this.defaultText = args.text || '';
            this.inherited(arguments);
        },
        
        /**
         * set a field on this component
         */
		setField: function( field, value ) {
			this.set(field, value);
		},
		
		/**
         * get a field on this component
         */
        getField: function( field ) {
			return this[field];
        },
		
		/**
		 * Return an object representing the state of this component to save in User Preferences 
		 */
        getPreferenceValue: function() {
			return {
				'value': this.get('value')
			};	
		},
		
		/**
		 * Clear the value of this component 
		 */
		clear: function() {
			this.set('value', this.defaultText);
		},

        render: function(d_parent) {
            /* these two lines are here because dijit/form/textbox does not support the HTML native textbox attribute "size" */
            domAttr.set(this.textbox, 'size', this.size);
            domClass.add(this.textbox, 'clearTextBox');

            /**
             * sometimes text boxes will have a pre-defined size, other times they will not. So, the useSize
             * flag can be used to determine that
             */
            if(this.useSize) {
                domStyle.set(this.domNode, 'width', this.size/1.5 + 'em');
            }
            else {
                domStyle.set(this.domNode, 'width', '100%');
            }

            // Add widget class to the domNode
            var domNode = this.textbox;
            // Create the "X" link
            this.clearLink = domConstruct.toDom('<a></a>');
            domClass.add(this.clearLink, 'clearTextBoxLink');
            domConstruct.place(this.clearLink, this.domNode, 'first');

            var that = this;
            on(this.clearLink, 'click', function() {
                that.clear();
                that.textbox.focus();
                that.keyUpCallback();
            });

            on(this, 'change', function() { that._checkValue(); });

            this.inherited(arguments);
        },

        _checkValue: function() {
            if(this.clearLink) {
                if(this.getField('value') === '') {
                    domStyle.set(this.clearLink, 'display', 'none');
                } else {
                    domStyle.set(this.clearLink, 'display', 'block');
                }
            }
        },

        startup: function() {
            // hide clear button if there is no text initially
            this._checkValue();
            this.inherited(arguments);
        },

		/**
		 * Defines a function to be used to determine if the passed-in value is within the bounds of the this components current value
		 */
		computeFilterLogic: function( value ) {
			var result = true;
			
			var blankIsFalse = false;
			if( this.snamode === 'ASPECT' ) {
				blankIsFalse = true;
			}
			
			var preferenceValue = this.getPreferenceValue().value;

			if( blankIsFalse === true && preferenceValue === '' ) {
				result = false;
			}
			else if( value && value !== '' ) {
                result = value.toLowerCase().indexOf(preferenceValue.toLowerCase()) !== -1;
			}
			
			return result;
		},

        destroyFacet: function() {
            this.inherited(arguments);

            // necessary because global search facet doesn't stay inside its container
            if(this.textbox) {
                this.destroyRecursive();
            }
        },
		
		showErrorState: function( message ) {
        	this._set("state", "Error");
        	this.showToolTip(message);
        }
    });
});
