/**
 * @class model.widgets.SessionMonitorDialog models the dialog that displays when the user will be logged out soon
 */
define([
    'dojo/on',
    'dojo/query',
    'dijit/registry',
    'dojo/parser',
    'dojo/dom-construct',
    'dijit/layout/ContentPane',
    '../../../util/I18nUtil',
    'dojo/_base/declare',
    'dijit/Dialog',
    'dojo/text!./templates/sessionmonitormodel.html'
], function(on, query, registry, parser, domConstruct, ContentPane, I18nUtil, declare, Dialog, template) {
    return declare([Dialog], {
        constructor: function( args ) {
            this.controller = args.controller;
        },

        startup: function() {
            var that = this;

            this.inherited(arguments);

            var node = domConstruct.toDom(template);
            parser.parse(node);

            domConstruct.place(node, this.containerNode);

            var yes = I18nUtil.get('global_yes');
            var no = I18nUtil.get('global_no');
            var title = I18nUtil.get('session_timeout_warning_title');

            var d_yes = registry.byId('sessionmonitor-yes');
            var d_no = registry.byId('sessionmonitor-no');
            var warningNode = query('span', node)[0];

            d_yes.set('label', yes);
            d_no.set('label', no);
            this.set('title', I18nUtil.get('helpabout_header_sna'));

            var continuePolling = true;
            var updater = function() {
                warningNode.innerHTML = I18nUtil.get('session_timeout_warning', that.controller.getInactiveTime());

                if(continuePolling) {
                    setTimeout(updater, 1000);
                }
            };
            updater();

            on(d_yes, 'click', function() {
                that.controller.resetSessionStatus();
                that.destroyRecursive();
                continuePolling = false;
            });

            on(d_no, 'click', function() {
                that.controller.logoff();
                that.destroyRecursive();
                continuePolling = false;
            });
        }
    });
});
