/**
 * @description 
 * 
 * @class
 */
define([
    'graphics/gtl/core/GTLStateful',
    'sna/networkview/controller/util/UndoStackHelper',
	'sna/networkview/controller/util/GroupsHelper',
	'dijit/registry',
    '../../model/util/NLDHelper',
    'graphics/gtl/core/nodeLinkDiagram/LinkDirectionEnum',
    'commons/collections/ArrayCollection',
    'graphics/gtl/core/GTLStateful',
    'graphics/gtl/core/nodeLinkDiagram/NodeLinkDiagramItemTypeEnum',
    'dojo/_base/lang',
	'dojo/_base/declare',
	'sna/util/DateUtil',
	'sna/util/ArrayUtil',
	'dojox/lang/functional'
], function(GTLStateful, UndoStackHelper, GroupsHelper, registry, NLDHelper, LinkDirectionEnum, ArrayCollection, Stateful, NodeLinkDiagramItemTypeEnum, lang, declare, DateUtil, ArrayUtil, functional ) {
	var statefulArray = new ArrayCollection([]);
	
	var getStatefulEntity = function( entityUid ) {
		var entity = null;
		for( var i = 0; i < statefulArray.length; i++ ) {
			if( statefulArray.getItemAt(i).get('uid') === entityUid ) {
				entity = statefulArray.getItemAt(i);
				break;
			}
		}
		
		return entity;
	};
	
	/**
	 * Start defining NetworkStatefulHelper
	 */
	var helper = declare(null, {
    });
	
	helper.sync = function( entityArray ) {
		var that = this;
		functional.forIn(entityArray, function( entity, entityIndex ) {
			var statefulEntity = getStatefulEntity(entity.uid);
			
			if( entity.type === 'NODE' ) {
                if(entity.borderColor) {
                    NLDHelper.addToColorModel('borderColor', entity.borderColor);
                }

				if( statefulEntity ) {
					var origNodeSymbol = statefulEntity.get('nodeSymbol') || 'empty';
					
					if( entity.positionX ) {
						statefulEntity.set('positionX', entity.positionX);
					}
					if( entity.positionY ) {
						statefulEntity.set('positionY', entity.positionY);
					}
					
					if( entity.degree || (entity && entity.degree === 0) ) {
						statefulEntity.set('nodeDegrees', entity.degree);
					}
					
					if( entity.nodeType ) {
						statefulEntity.set('nodeType', entity.nodeType);
					}
					
					if( entity.nodeSymbol ) {
                        statefulEntity.set('nodeSymbol', entity.nodeSymbol);
                    }

                    if( entity.nodeLabel ) {
                        statefulEntity.set('nodeLabel', entity.nodeLabel);
                    }
                    else {
                        statefulEntity.set('nodeLabel', '');
                    }

                    if(entity.borderColor) {
                        statefulEntity.set('borderColor', entity.borderColor);
                    }
                    else {
                        statefulEntity.set('borderColor', '0x000000');
                    }

                    if(entity.borderSize) {
                        statefulEntity.set('borderSize', entity.borderSize);
                    }

                    if(entity.nodeColor) {
                        statefulEntity.set('nodeColor', entity.nodeColor);
                    }

					if( entity.highlight === true || entity.highlight === false ) {
						statefulEntity.set('highlight', entity.highlight);
					}
				}
				else {
					statefulArray.addItem(new Stateful({
						uid: entity.uid, 
						type: NodeLinkDiagramItemTypeEnum.NODE, 
						positionX: entity.positionX, 
						positionY: entity.positionY, 
						nodeType: entity.nodeType,
						nodeSymbol: entity.evolutions2[0].nodeSymbol,
						nodeDegrees: entity.nodeDegrees || 0,
                        nodeLabel: entity.nodeLabel,
                        borderColor: entity.borderColor || '0x000000',
                        nodeColor: entity.nodeColor || '0x000000'
					}));
				}
			}
			else if( entity.type === 'LINK' ) {
                if(entity.linkColor) {
                    NLDHelper.addToColorModel('linkColor', entity.linkColor);
                }

				if( statefulEntity ) {
					statefulEntity.set('fromNodeId', entity.fromNodeId);
					statefulEntity.set('toNodeId', entity.toNodeId);
					statefulEntity.set('linkStartDt', entity.linkStartDt);
					statefulEntity.set('linkEndDt', entity.linkEndDt);
                    statefulEntity.set('linkWidth', entity.linkWidth);
                    statefulEntity.set('linkDirection', entity.linkDirection);
                    statefulEntity.set('linkColor', entity.linkColor);
                    statefulEntity.set('linkStyle', entity.linkStyle);

                    if( entity.highlight === true || entity.highlight === false ) {
                        statefulEntity.set('highlight', entity.highlight);
                    }

				}
				else {
					statefulArray.addItem(new Stateful({
						uid: entity.uid,
						type: NodeLinkDiagramItemTypeEnum.LINK,
						fromNodeId: entity.fromNodeId,
						toNodeId: entity.toNodeId,
						linkStartDt: entity.linkStartDt || 0,
						linkEndDt: entity.linkEndDt || 0,
                        linkWidth: entity.linkWidth || 2,
                        highlight: entity.highlight || false,
                        linkDirection: entity.linkDirection || LinkDirectionEnum.NONE,
                        linkColor: entity.linkColor || '0x808080',
                        linkStyle: entity.linkStyle || '1'
					}));
				}
			}
            else if( entity.type === 'GROUP' ) {
                if( statefulEntity ) {
                    if( entity.highlight === true || entity.highlight === false ) {
                        statefulEntity.set('highlight', entity.highlight);
                    }
                } else {
                    statefulArray.addItem(new GTLStateful({
                        borderSize: entity.borderSize,
                        groupColor: entity.groupColor,
                        groupLabel: entity.groupLabel,
                        groupSymbol: entity.groupSymbol,
                        nodeUid: entity.nodeUid,
                        type: NodeLinkDiagramItemTypeEnum.GROUP,
                        nodeSymbol: GroupsHelper.GROUPICON,
                        uid: entity.uid
                    }));
                }
            }
		});
	};

    helper.handleRawGroups = function( rawJson ) {
    	var groupsMap = GroupsHelper.generateGroupsMap(rawJson);

        functional.forIn(groupsMap, function( groupMemberArray, groupIndex ) {
            helper.addGroup(groupIndex, groupMemberArray);
        });
    };

    helper.addGroup = function( groupUid, jsonGroupArray ) {
        var statefulGroupMemberArray = [];
        functional.forIn(jsonGroupArray, function( groupMember, groupMemberIndex ) {
            var statefulEntity = helper.getStatefulObjectFromUid(groupMember.nodeUid);

            if( statefulEntity ) {
                statefulGroupMemberArray.push(statefulEntity);
            }
        });

        NLDHelper.createNewGroup(groupUid, statefulGroupMemberArray);

        var groupUids = [];
        for( var i = 0; i < jsonGroupArray.length; i++ ) {
 			var jsonGroupObject = jsonGroupArray[i];

 			groupUids.push(jsonGroupObject.nodeUid);
        }
		UndoStackHelper.clearStackIfContains(UndoStackHelper.GROW, groupUids);
		UndoStackHelper.clearStackIfContains(UndoStackHelper.REGROUP, groupUids);
    };

    helper.removeGroup = function( groupUid ) {
        var statefulGroup = helper.getStatefulObjectFromUid(groupUid);
        if (statefulGroup.type === NodeLinkDiagramItemTypeEnum.GROUP) {
            var idx = statefulArray.getItemIndex(statefulGroup);
            var childrenUids = [];

            if (idx >= 0) {
                //Remove the group item
                statefulArray.removeItemAt(idx);

                //Reset children
                var children = statefulGroup.children;
                for (var i = children.length - 1; i >= 0; i--) {
                	childrenUids.push(children[i].get('uid'));
                    children[i].group = null;
                }

                // enable regroup nodes button
        		registry.byId('snaRegroupNodesButton').set('disabled', false);

        		UndoStackHelper.push(UndoStackHelper.REGROUP, {
        			keyUid: groupUid,
        			uids: childrenUids
        		});
            }
        }
	};

    helper.getStatefulObjectFromUid = function( uid ) {
        var statefulObject;

        for( var i = 0; i < statefulArray.length; i++ ) {
            var item = statefulArray.getItemAt(i);

            if( item.get('uid') === uid ) {
                statefulObject = item;
                break;
            }
        }

        return statefulObject;
    };
	
	helper.getArrayIndex = function( uid ) {
		var arrayIndex = 0;
		
		for( var i = 0; i < statefulArray.length; i++ ) {
			var statefulEntity = statefulArray.getItemAt(i);
			if( uid === statefulEntity.get('uid') ) {
				arrayIndex = i;
				break;
			}
		}
		
		return arrayIndex;
	};
	
	helper.removeEntity = function( uid ) {
		var arrayIndex = helper.getArrayIndex(uid);
		statefulArray.removeItemAt(arrayIndex);
	};
	
	helper.reverseSync = function( subsetNodes ) {
		functional.forIn(subsetNodes, function( node, nodeIndex ) {
			var statefulEntity = getStatefulEntity(node.uid);
			
			node.positionX = statefulEntity.get('positionX');
			node.positionY = statefulEntity.get('positionY');
			
			console.debug(node.uid + ': ' + node.positionX + ' ' + node.positionY);
		});
	};
	
	helper.getStatefulEntityArray = function() {
		return statefulArray;
	};
	
	helper.clear = function() {
		statefulArray.removeAll();
	};
	
	helper.entityFromIndex = function( index ) {
		return statefulArray.getItemAt(index); 
	};
	
	helper.entitiesFromIndicies = function( indicies ) {
		var entityArray = [];
		for( var i = 0; i < indicies.length; i++ ) {
			var currentIndex = indicies[i];
			
			var entity = helper.entityFromIndex(currentIndex);
			if( entity ) {
				entityArray.push(entity);
			}
		}
		
		return entityArray;
	};
	
	return helper;
});
