/**
 * @class 
 */
define([
    'dijit/ColorPalette',
    'dijit/form/HorizontalRule',
    'dijit/form/HorizontalRuleLabels',
    'dijit/registry',
    'dojo/_base/array',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/dom',
    'dojo/dom-attr',
    'dojo/dom-construct',
    'dojo/dom-construct',
    'dojo/dom-style',
    'dojo/on',
    'dojo/parser',
    'dojo/query',
    'dojo/ready',
    'dojo/text!src/sna/resources/html/models/networkview.html',
    'dojox/lang/functional',
    'dojox/widget/Standby',
    'sna/model/widgets/LoadingSpinner/LoadingSpinner',
    'sna/networkview/controller/util/GroupsHelper',
    'sna/networkview/controller/util/NetworkStatefulHelper',
    'sna/networkview/controller/util/NetworkValidator',
    'sna/networkview/controller/util/TimeSliderHelper',
    'sna/networkview/controller/util/UndoStackHelper',
    'sna/networkview/model/util/NLDHelper',
    'sna/networkview/model/widgets/EditLinkDialog/EditLinkDialog',
    'sna/networkview/model/widgets/EditNodeDialog/EditNodeDialog',
    'sna/networkview/model/widgets/TabbedDialogContainer/TabbedDialogContainer',
    'sna/util/ContextPreferenceUtil',
    'sna/util/DateUtil',
    'sna/util/I18nUtil',
    'sna/util/OverlayUtil'
], function( ColorPalette, HorizontalRule, HorizontalRuleLabels, registry, array, declare, lang, dom, domAttr, construct, domConstruct,
		    domStyle, on, parser, query, ready, networkViewTemplate, functional, Standby,
		    LoadingSpinner, GroupsHelper, NetworkStatefulHelper, NetworkValidator, TimeSliderHelper, UndoStackHelper, NLDHelper, EditLinkDialog, EditNodeDialog, TabbedDialogContainer, ContextPreferenceUtil, DateUtil, I18nUtil, overlayUtil ) {

	var performButtonAction = function( d_button, callback ) {
		var disabled = d_button.get('disabled'); 
		if( disabled === false ) {
			callback();
		}
	};

	var _refreshGraph = function() {
		NLDHelper.refresh(this.graph, this.node);
	};
	
	var _entityClicked = function( entityId ) {
		this.controller.setFocusedEntityId(entityId);
		this.controller.showPropertiesSection();
	};
	
	var triggerTimeSlider = function( value ) {
		this.controller.filterDataFromTimeSliderValue(value);
	};
	
	var _handleI18N = function() {
		var d_snaSaveButton = registry.byId('snaSaveButton');
		var d_snaLoadNetworkDropdownButton = registry.byId('snaLoadNetworkDropdownButton');
		var d_snaLoadNetworkDropdownButton_fromstp = registry.byId('snaLoadNetworkDropdownButton_fromstp');
		var d_snaLoadNetworkDropdownButton_db = registry.byId('snaLoadNetworkDropdownButton_db');
		var d_snaAddLinkButton = registry.byId('snaAddLinkButton');
		var d_snaAddNodeButton = registry.byId('snaAddNodeButton');
		var d_snaEditEntityButton = registry.byId('snaEditEntityButton');
		var d_snaRemoveEntityButton = registry.byId('snaRemoveEntityButton');
		var d_snaGroupNodesButton = registry.byId('snaGroupNodesButton');
		var d_snaRegroupNodesButton = registry.byId('snaRegroupNodesButton');
		var d_snaCollapseExpandedNodesButton = registry.byId('snaCollapseExpandedNodesButton');
		var d_openStreetMapsButton = registry.byId('openStreetMapsButton');
		var d_hideStreetMapsButton = registry.byId('hideStreetMapsButton');
		var d_homedirectionalbutton = registry.byId('homedirectionalbutton');
		var d_homedirectionalbuttonmenuleft = registry.byId('homedirectionalbuttonmenuleft');
		var d_homedirectionalbuttonmenuright = registry.byId('homedirectionalbuttonmenuright');
		var d_homedirectionalbuttonmenuup = registry.byId('homedirectionalbuttonmenuup');
		var d_homedirectionalbuttonmenudown = registry.byId('homedirectionalbuttonmenudown');
		var d_snaZoomInButton = registry.byId('snaZoomInButton');
		var d_snaZoomOutButton = registry.byId('snaZoomOutButton');
		var d_snaRunLayoutAlgorithmDropdownButton = registry.byId('snaRunLayoutAlgorithmDropdownButton');
		var d_snaRunLayoutAlgorithmDropdownButton_hierarchical = registry.byId('snaRunLayoutAlgorithmDropdownButton_hierarchical');
		var d_snaRunLayoutAlgorithmDropdownButton_fr1 = registry.byId('snaRunLayoutAlgorithmDropdownButton_fr1');
		var d_snaRunLayoutAlgorithmDropdownButton_fr2 = registry.byId('snaRunLayoutAlgorithmDropdownButton_fr2');
		var d_snaRunLayoutAlgorithmDropdownButton_fr3 = registry.byId('snaRunLayoutAlgorithmDropdownButton_fr3');
		
		d_snaSaveButton.set('label', I18nUtil.get('networkview_savenetwork'));
		d_snaLoadNetworkDropdownButton.set('label', I18nUtil.get('bizdomain_open'));
		d_snaLoadNetworkDropdownButton_fromstp.set('label', I18nUtil.get('networkview_fromstoredprocess'));
		d_snaLoadNetworkDropdownButton_db.set('label', I18nUtil.get('networkview_lastsavedtodatabase'));
		d_snaAddLinkButton.set('label', I18nUtil.get('networkview_addlink'));
		d_snaAddNodeButton.set('label', I18nUtil.get('networkview_addnode'));
		d_snaEditEntityButton.set('label', I18nUtil.get('networkview_editentity'));
		d_snaRemoveEntityButton.set('label', I18nUtil.get('networkview_deleteentity'));
		d_snaGroupNodesButton.set('label', I18nUtil.get('networkview_groupnodes'));
		d_snaRegroupNodesButton.set('label', I18nUtil.get('networkview_regroupnodes'));
		d_snaCollapseExpandedNodesButton.set('label', I18nUtil.get('networkview_collapseexpandednodes'));
		d_openStreetMapsButton.set('label', I18nUtil.get('networkview_showmap'));
		d_hideStreetMapsButton.set('label', I18nUtil.get('networkview_hidemap'));
		d_homedirectionalbutton.set('title', I18nUtil.get('networkview_home'));
		d_homedirectionalbuttonmenuleft.set('title', I18nUtil.get('networkview_panleft'));
		d_homedirectionalbuttonmenuright.set('title', I18nUtil.get('networkview_panright'));
		d_homedirectionalbuttonmenuup.set('title', I18nUtil.get('networkview_panup'));
		d_homedirectionalbuttonmenudown.set('title', I18nUtil.get('networkview_pandown'));
		d_snaZoomInButton.set('label', I18nUtil.get('networkview_zoomin'));
		d_snaZoomOutButton.set('label', I18nUtil.get('networkview_zoomout'));
		d_snaRunLayoutAlgorithmDropdownButton.set('label', I18nUtil.get('networkview_runlayout'));
		d_snaRunLayoutAlgorithmDropdownButton_hierarchical.set('label', I18nUtil.get('networkview_treelayout'));
		d_snaRunLayoutAlgorithmDropdownButton_fr1.set('label', I18nUtil.get('networkview_springy'));
		d_snaRunLayoutAlgorithmDropdownButton_fr2.set('label', I18nUtil.get('networkview_morespringy'));
		d_snaRunLayoutAlgorithmDropdownButton_fr3.set('label', I18nUtil.get('networkview_mostspringy'));
	};
	
	/**
	 * _eventHandlers
	 * 
	 * define event handlers
	 */
	var _eventHandlers = function() {
		var that = this;
        
        // when the side content pane's are collapsed/expanded, resize the network component
        on(query('#networkviewfacetscollapsebutton'), 'click', function() {
        	lang.hitch(that, _refreshGraph)();
        });
        on(query('#networkviewpropertiescollapsebutton'), 'click', function() {
        	lang.hitch(that, _refreshGraph)();
        });
        
        on(query('#snaSaveButton'), 'click', function() {
            var disabled = registry.byId('snaSaveButton').get('disabled');
            if( !disabled ) {
        		that.controller.saveNetwork();
        	}
        });
        
        on(query('#homedirectionalbutton'), 'click', function() {
            performButtonAction(registry.byId('homedirectionalbutton'), function() {
		        NLDHelper.center();
            });
        });
        
        on(query('#snaZoomInButton'), 'click', function() {
            performButtonAction(registry.byId('snaZoomInButton'), function() {
		        NLDHelper.zoomIn();
            });
        });
        on(query('#snaZoomOutButton'), 'click', function() {
            performButtonAction(registry.byId('snaZoomOutButton'), function() {
		        NLDHelper.zoomOut();
            });
        });

        on(query('#snaGroupNodesButton'), 'click', function() {
            performButtonAction(registry.byId('snaGroupNodesButton'), function() {
                var currentlySelectedStatefulEntities = NLDHelper.getSelected();
                var selectedUids = [];
                functional.forIn(currentlySelectedStatefulEntities, function( statefulEntity ) {
                    selectedUids.push(statefulEntity.get('uid'));
                });

                that.controller.addGroup(null, selectedUids);
            });
        });
        on(query('#snaRegroupNodesButton'), 'click', function() {
            performButtonAction(registry.byId('snaRegroupNodesButton'), function() {
                var isSafe = GroupsHelper.isUndoOperationSafe(that.controller.networkState);
                if( isSafe === true ) {
                    var undoGroupStackObject = UndoStackHelper.pop(UndoStackHelper.REGROUP);
                    that.controller.addGroup(undoGroupStackObject.keyUid, undoGroupStackObject.uids);
                }
                else {
                    console.debug('regroup not safe');
                }
            });
        });
        on(query('#snaCollapseExpandedNodesButton'), 'click', function() {
            performButtonAction(registry.byId('snaCollapseExpandedNodesButton'), function() {
            	that.controller.collapseExpandedNodes();
            });
        });
        on(query('#snaAddLinkButton'), 'click', function() {
            performButtonAction(registry.byId('snaAddLinkButton'), function() {
                that.controller.startAddLinkActions();
            });
        });
        on(query('#snaAddNodeButton'), 'click', function() {
            performButtonAction(registry.byId('snaAddNodeButton'), function() {
                that.controller.startAddNodeActions();
            });
        });
        on(query('#snaRemoveEntityButton'), 'click', function() {
            performButtonAction(registry.byId('snaRemoveEntityButton'), function() {
                that.controller.startRemoveEntityActions();
            });
        });
        
        on(query('#snaEditEntityButton'), 'click', function() {
            performButtonAction(registry.byId('snaEditEntityButton'), function() {
                that.controller.startEditEntityActions();
            });
        });

        // since the global search textbox is several different things, we need to hide them when
        // we switch between the sna tab and the details tab
        on(registry.byId('snabordercontainer'), 'show', function() {
            that.controller.networkFacetController.showGlobalSearchFacet();
        });

        on(registry.byId('snabordercontainer'), 'hide', function() {
            that.controller.networkFacetController.hideGlobalSearchFacet();
        });

        // directional button click handlers
        var d_homedirectionalbuttonmenuleft = registry.byId('homedirectionalbuttonmenuleft').domNode;
        on(d_homedirectionalbuttonmenuleft, 'click', function() {
        	NLDHelper.panLeft();
        });
        var d_homedirectionalbuttonmenuright = registry.byId('homedirectionalbuttonmenuright').domNode;
        on(d_homedirectionalbuttonmenuright, 'click', function() {
        	NLDHelper.panRight();
        });
        var d_homedirectionalbuttonmenuup = registry.byId('homedirectionalbuttonmenuup').domNode;
        on(d_homedirectionalbuttonmenuup, 'click', function() {
        	NLDHelper.panUp();
        });
        var d_homedirectionalbuttonmenudown = registry.byId('homedirectionalbuttonmenudown').domNode;
        on(d_homedirectionalbuttonmenudown, 'click', function() {
        	NLDHelper.panDown();
        });
        
        // cumulative and marginal buttons
        registry.byId("cumulative").on("change", function( isChecked ){
        	var d_networkTimeSlider = registry.byId('timeslidercontainer');
        	
        	//var timeSliderValue = d_networkTimeSlider.get('value');
        	var timeSliderValue = d_networkTimeSlider.get('value');
        	
            lang.hitch(that, triggerTimeSlider, timeSliderValue)();
        }, true);
        
        
        // reload network buttons
        registry.byId('snaLoadNetworkDropdownButton_fromstp').on("click", function() {
        	that.controller.reloadNetwork('NETWORK_SOURCE_STOREDPROCESS');
        });
        registry.byId('snaLoadNetworkDropdownButton_db').on("click", function() {
        	that.controller.reloadNetwork('NETWORK_SOURCE_DATABASE');
        });

        // layout function to start and end the loading spinner
        var layoutRunner = function(layout, options) {
            var isDone = false;
            var completionCallback = function () { return isDone;};
            new LoadingSpinner().target(registry.byId('snadiagramcontentpane')).until(completionCallback).startSpinners(function(){});
            that.controller.startRunLayoutAlgorithmActions(layout, options, function () { isDone = true;});
        };

        // run layout
        registry.byId('snaRunLayoutAlgorithmDropdownButton_hierarchical').on('click', function () {
            layoutRunner('HierarchicalLayout', null);
        });
        registry.byId('snaRunLayoutAlgorithmDropdownButton_fr1').on('click', function () {
            layoutRunner('FRLayout', {
                iterations: 400
            });
        });
        registry.byId('snaRunLayoutAlgorithmDropdownButton_fr2').on('click', function () {
            layoutRunner('FRLayout', {
                iterations: 4000
            });
        });
        registry.byId('snaRunLayoutAlgorithmDropdownButton_fr3').on('click', function () {
            layoutRunner('FRLayout', {
                iterations: 40000
            });
        });

        // show network info
        on(query('#snaShowNetworkInfoButton'), 'click', function () {
            lang.hitch(that.controller, that.controller.showNetworkInfoButtonClicked)();
        });
    }; // end of _eventHandlers
    
    var showTabbedDialog = function( contentArray, eventMap ) {
		var showDialog = function( dialog ) {
			var d_loadingArea = domConstruct.create('div', {
	        }, dom.byId('container'), 'after');
			
			dialog.placeAt(d_loadingArea);
			dialog.showDialog();
		};
		
		var tabbedDialogContainer = new TabbedDialogContainer({
			eventMap: eventMap,
			title: I18nUtil.get('networkview_graphentitieswizard')
		});
    	tabbedDialogContainer.populate(contentArray);
		showDialog(tabbedDialogContainer);
		
		return tabbedDialogContainer;
	};

    var getSafeTimesliderFields = function() {
        var d_networkTimeSlider = registry.byId('timeslidercontainer');

        var rawMax = d_networkTimeSlider.get('maximum');
        var rawMin = d_networkTimeSlider.get('minimum');

        var safeMax = DateUtil.getSASEpochDateObject();
        if( rawMax ) {
            safeMax = DateUtil.fromSasDate(rawMax);
        }

        var safeMin = DateUtil.getSASEpochDateObject();
        if( rawMin ) {
            safeMin = DateUtil.fromSasDate(rawMin);
        }

        return {
            maximum: safeMax,
            minimum: safeMin
        };
    };
    
    return declare(null, {
    	/**
    	 * Constructor
    	 */
        constructor: function(args) {
            this.controller = args.controller;
            this.loaded = false;
        },
        
        /**
         * load
         */
        load: function( loadedCallback ) {
        	var that = this;
    
        	// handle loading screen
        	if( that.loaded === false ) {
        		var snaBorderContainerStandby = new Standby({target: "snabordercontainer"});
            	document.body.appendChild(snaBorderContainerStandby.domNode);
	            snaBorderContainerStandby.startup();
	            snaBorderContainerStandby.show();
            }
        	
        	ready(function () {
        		if( that.loaded === false ) {
        			var d_snabordercontainer = registry.byId('snabordercontainer').domNode;
	        		domConstruct.empty(d_snabordercontainer);
	                domConstruct.place(networkViewTemplate, d_snabordercontainer);
	                parser.parse(d_snabordercontainer);
	                registry.byId('snabordercontainer').resize();
	                snaBorderContainerStandby.hide();
	                overlayUtil.hideOverlay();
	            }
        		
        		if( loadedCallback ) {
        			loadedCallback();
        		}
        		
                registry.byId('snaRegroupNodesButton').set('disabled', 'disabled');
                registry.byId('snaCollapseExpandedNodesButton').set('disabled', 'disabled');
        		lang.hitch(that, _eventHandlers)();
        		lang.hitch(that, _handleI18N)();
                registry.byId('snaCollapseExpandedNodesButton').set('disabled', 'disabled');
        		
                that.loaded = true;
                
                I18nUtil.parseDOM();
        	});
        },
        
        /**
         * setup TimeSlider with the passed-in array of values
         */
        setupTimeSlider: function( timeSliderValues ) {
        	var that = this;
        	var d_networkTimeSlider = registry.byId('timeslidercontainer');
        	d_networkTimeSlider.setValues(timeSliderValues);
        	d_networkTimeSlider.setCallbacks({
        		onChange: function( value ) {
        		    var timeSliderValue = d_networkTimeSlider.get('value');
        		    
        			lang.hitch(that, triggerTimeSlider, timeSliderValue)();
        		}
        	});
        },
        
        updateTimeSlider: function( timeSliderValues ) {
        	var d_networkTimeSlider = registry.byId('timeslidercontainer');
        	d_networkTimeSlider.updateValues(timeSliderValues);
        },
        
        getTimeSliderValue: function() {
        	var d_networkTimeSlider = registry.byId('timeslidercontainer');
        	return d_networkTimeSlider.get('value');
        },
        
        /**
         * cleanupView
         */
        cleanupView: function() {
        	this.loaded = false;
        },

        /**
         * render
         */
        render: function (statefulEntityArray, plugins, vertical, d_parentId, callback) {
            var that = this;
            ready(function () {
            	// setup node-link component
            	var d_nodeLinkContainer = registry.byId('snadiagramcontentpane').domNode;
            	that.node = NLDHelper.prepare(d_nodeLinkContainer);

                // start nld component, and define event callbacks
            	NLDHelper.start(that.controller, statefulEntityArray, that.node, {
            		entitySingleClick: function( uid ) {
            		    if( uid ) {
                            var previousEntityId = that.controller.getFocusedEntityId();
            		    	that.controller.setFocusedEntityId(uid);
            		    	
            		    	// if properties is already open, single click updates it with the just-clicked entity
            		    	if( that.controller.isPropertiesSectionOpen() ) {
            		    		that.controller.showPropertiesSection(previousEntityId);
            		    	}
                		}
            		},
            		
            		entityDoubleClick: function( uid ) {
            		    if( uid ) {
                            var previousEntityId = that.controller.getFocusedEntityId();
                            that.controller.setFocusedEntityId(uid);
            		    	that.controller.showPropertiesSection();
                		}
            		},

                    multipleEntitiesSelected: function( uids ) {
                        that.controller.hideNodeSpecificSections();
                    }
            	},
            	  // the callback from NLDHelper.start
            	  function( graph ) {
            		if( callback ) {
            			that.graph = graph;
                		callback();
                	}
            	});
            });
        },
        
        /**
         *
         */
        updateAsideText: function() {
            NLDHelper.displayAsideText();
        },
        
        /**
         * 
         */
        showAddNodeDialog: function( networkState, fromNode, emptyNode, emptyLink, nodeIsReadyForActionCallback ) {
        	var that = this;

            var safeTimeSliderFields = getSafeTimesliderFields();

            var protoLink = {};
            lang.mixin(protoLink, emptyLink);
            protoLink.linkStartDt = safeTimeSliderFields.minimum;
            protoLink.linkEndDt = safeTimeSliderFields.maximum;

        	var d_linkDialog = new EditLinkDialog({
        		title: I18nUtil.get('networkview_addlink'),
        		link:  protoLink,
        		nodes: networkState.nodes,
        		fromNode: fromNode,
        		toNode: emptyNode,
        		newLink: true,
        		mode: 'ADDNODE'
        	});
        	var d_nodeDialog = new EditNodeDialog({
        		title: I18nUtil.get('networkview_addnode'),
        		node: emptyNode,
        		newNode: true,
        		uidChanged: function( newUid ) {
        		    d_linkDialog.setToNodeUid(newUid);
        		}
        	});
        	
        	var dialogArray = [];
        	dialogArray.push(d_nodeDialog);
        	dialogArray.push(d_linkDialog);
        	
        	var tabbedDialog = showTabbedDialog(dialogArray, {
        		onSubmit: function( payloadObjectArray ) {
	        		var proposedNode = {};
	        		var proposedLink = {};
	        		
	        		if( payloadObjectArray[0].type === 'NODE' ) {
	        			proposedNode = lang.mixin(emptyNode, payloadObjectArray[0]);
	        			proposedLink = lang.mixin(emptyLink, payloadObjectArray[1]);
	        		}
	        		else if( payloadObjectArray[0].type === 'LINK' ) {
	        			proposedNode = lang.mixin(emptyNode, payloadObjectArray[1]);
	        			proposedLink = lang.mixin(emptyLink, payloadObjectArray[0]);
	        		}
	        		
	        		var nodeValidationResults = NetworkValidator.validateNode(proposedNode);
	        		var linkValidationResults = NetworkValidator.validateLink(proposedLink);
	        		
	    			if( nodeValidationResults.length === 0 && linkValidationResults.length === 0 ) {
	    				nodeIsReadyForActionCallback(proposedNode, proposedLink);
	        			
	    				tabbedDialog.end();
	    			}
	    			else {
	    				d_nodeDialog.showValidationResults(nodeValidationResults);
	    				d_linkDialog.showValidationResults(linkValidationResults);
	    			}
        		}
        	});
        },
        
        /**
         * 
         */
        showAddLinkDialog: function( networkState, fromNode, toNode, emptyLink, linkIsReadyForActionCallback ) {
        	var that = this;

            var safeTimeSliderFields = getSafeTimesliderFields();
            var protoLink = {};
            lang.mixin(protoLink, emptyLink);
            protoLink.linkStartDt = safeTimeSliderFields.minimum;
            protoLink.linkEndDt = safeTimeSliderFields.maximum;
        	
        	var d_linkDialog = new EditLinkDialog({
        		title: I18nUtil.get('networkview_addlink'),
        		link:  protoLink,
        		nodes: networkState.nodes,
        		fromNode: fromNode,
        		toNode: toNode,
                newLink: true,
        		mode: 'ADD'
        	});
        	
        	var dialogArray = [d_linkDialog];
        	
        	var tabbedDialog = showTabbedDialog(dialogArray, {
        		onSubmit: function( payloadObjectArray ) {
	        		var proposedLink = lang.mixin(emptyLink, payloadObjectArray[0]);
	        		
	        		var linkValidationResults = NetworkValidator.validateLink(proposedLink);
	        		
	    			if( linkValidationResults.length === 0 ) {
	    				linkIsReadyForActionCallback(proposedLink, proposedLink);
	        			
	    				tabbedDialog.end();
	    			}
	    			else {
	    				d_linkDialog.showValidationResults(linkValidationResults);
	    			}
        		}
        	});
        },
        
        /**
         * 
         */
        showEditNodeDialog: function( node, nodeIsReadyForActionCallback ) {
        	var that = this;
        	
        	var d_nodeDialog = new EditNodeDialog({
        		title: I18nUtil.get('networkview_addnode'),
        		node: node
        	});
        	
        	var dialogArray = [d_nodeDialog];
        	
        	var tabbedDialog = showTabbedDialog(dialogArray, {
        		onSubmit: function( payloadObjectArray ) {
	        		var proposedNode = lang.mixin(node, payloadObjectArray[0]);
	        		
	        		var nodeValidationResults = NetworkValidator.validateNode(proposedNode);
	    			if( nodeValidationResults.length === 0 ) {
	    				nodeIsReadyForActionCallback(proposedNode);
	        			
	    				tabbedDialog.end();
	    			}
	    			else {
	    				d_nodeDialog.showValidationResults(nodeValidationResults);
	    			}
        		}
        	});
        },
        
        /**
         * 
         */
        showEditLinkDialog: function( networkState, link, linkIsReadyForActionCallback ) {
        	var that = this;

        	var d_linkDialog = new EditLinkDialog({
        		title: I18nUtil.get('networkview_editentity'),
        		link: link,
        		nodes: networkState.nodes
        	});
        	
        	var dialogArray = [d_linkDialog];
        	
        	var tabbedDialog = showTabbedDialog(dialogArray, {
        		onSubmit: function( payloadObjectArray ) {
	        		var proposedLink = lang.mixin(link, payloadObjectArray[0]);
	        		
	        		var linkValidationResults = NetworkValidator.validateLink(proposedLink);
	    			if( linkValidationResults.length === 0 ) {
	    				linkIsReadyForActionCallback(proposedLink);
	        			
	    				tabbedDialog.end();
	    			}
	    			else {
	    				d_linkDialog.showValidationResults(linkValidationResults);
	    			}
        		}
        	});
        },
        
        /**
         * 
         */
        getTimeSliderAlgorithm: function() {
        	var timesliderAlgorithm = TimeSliderHelper.MARGINAL;
        	if( registry.byId("cumulative").get('checked') === true ) {
        		timesliderAlgorithm = TimeSliderHelper.CUMULATIVE;
        	}
        	
        	return timesliderAlgorithm;
        },
        
        isAlreadyLoaded: function() {
        	var loaded = false;
        	
        	if( query(".networkdiagram")[0] ) {
        		loaded = true;
	    	}
        	
        	return loaded;
        }
    });
});
