/**
 * @description 
 * 
 * @class
 */
define([
    'sna/networkview/controller/util/GroupsHelper',
    'sna/util/GUIDUtil',
    'graphics/gtl/core/styles/LineAttributes',
    'sna/util/UserPreferenceUtil',
    'graphics/gtl/core/nodeLinkDiagram/ViewportFitPolicyEnum',
    'graphics/gtl/core/VerticalAlignmentEnum',
    'graphics/gtl/core/HorizontalAlignmentEnum',
    'graphics/gtl/core/events/NodeLinkDiagramActionEvent',
    'dojo/query',
    'commons/collections/ArrayCollection',
    'commons/events/CollectionEvent',
    'dojo/Stateful',
    'dojo/_base/lang',
    'sna/networkview/model/util/NLDThemeUtil',
    'sna/util/IconUtil',
    'dojox/lang/functional',
    'commons/models/dataSet/RowEvent',
    'dijit/registry',
    'dojo/on',
    'dojo/dom-construct',
    'dojo/dom-style',
    'dojo/_base/declare',
    'graphics/gtl/core/styles/attributesMap/DiscreteAttributesMap',
    'graphics/gtl/core/styles/attributesMap/DiscreteAttributesMapEntry',
    'graphics/gtl/core/styles/FillAttributes',
    'graphics/gtl/core/nodeLinkDiagram/NodeLinkDiagramItemTypeEnum',
    'graphics/gtl/core/nodeLinkDiagram/LinkDirectionEnum',
    'graphics/gtl/data/ArrayDataProvider',
    'graphics/gtl/data/ArrayCollectionDataProvider',
    'graphics/gtl/core/ColorModel',
	'graphics/gtl/core/ColorModelTypeEnum',
    'graphics/gtl/layoutRegion/networkDiagram/NetworkDiagram',
    'graphics/gtl/layoutRegion/LayoutRegion',
    'graphics/gtl/Graph',
    'graphics/gtl/data/Role',
    'graphics/gtl/GraphView',
    'graphics/gtl/core/TriStateEnum'
], function( GroupsHelper, GUIDUtil, LineAttributes, UserPreferenceUtil, ViewportFitPolicyEnum, VerticalAlignmentEnum, HorizontalAlignmentEnum,
             NodeLinkDiagramActionEvent, query, ArrayCollection, CollectionEvent, Stateful, lang, NLDThemeUtil, IconUtil, functional,
		     RowEvent, registry, on, domConstruct, domStyle, declare, DiscreteAttributesMap, DiscreteAttributesMapEntry, FillAttributes,
		     NodeLinkDiagramItemTypeEnum, LinkDirectionEnum, ArrayDataProvider, ArrayCollectionDataProvider, ColorModel, ColorModelTypeEnum, 
		     NetworkDiagram, LayoutRegion, Graph, Role, GraphView, TriStateEnum ) {
	
	var moveIncrement = 10;
	
	var backgroundColor = '2F2F2F';
	
    // resize component so it fits it's container
    var _refresh = function( graph, node ) {
        this._graphView.setSize(node.clientWidth, node.clientHeight);

        // TODO this is a hack to work around the missing line in GraphView, should be fixed in d4htmcmn14
        this._graphView.graphRenderer.setActualSize(node.clientWidth, node.clientHeight);
    };

    var _getTypeStyles = function( callback ) {
    	var typeStyles = new DiscreteAttributesMap();
    	IconUtil.updateIconArrayCache(function( networkIconMap ) {
	    	functional.forIn(networkIconMap, function( iconObject, index ) {
	    		var dame = new DiscreteAttributesMapEntry();
	    		dame.values = iconObject.values;
	    		dame.icon = iconObject.filename;
	    		typeStyles.entries.push(dame);
	    	});
	    	
	    	callback(typeStyles);
    	});
    };

    var _mapIsVisible = function(map) {
        var divmap = document.getElementById("map");

        if(!divmap) {
            return false;
        }

        if (divmap.children.length > 0 && domStyle.get(divmap, 'display') !== 'none') {
            return divmap;
        }

        return false;
    };

    var _getScoreStyles = function() {
        var scoreStyles = new DiscreteAttributesMap();
        
    	// Build Score Styles
        var lowStyle = new DiscreteAttributesMapEntry();
        lowStyle.values = ['low'];
        lowStyle.fillAttributes = new FillAttributes();
        lowStyle.fillAttributes.setStyle("fillColor", "0xa0a0c0");
        //lowStyle.fillAttributes.setStyle("fillColor", "0xFFFFFF");

        var highStyle = new DiscreteAttributesMapEntry();
        highStyle.values = ['high'];
        highStyle.fillAttributes = new FillAttributes();
        highStyle.fillAttributes.setStyle("fillColor", "0xe11610");

        scoreStyles.entries = [lowStyle, highStyle];
        
        return scoreStyles;
    };
	
	/**
	 * Start defining NLDHelper
	 */
	var helper = declare(null, {
    });
	
	/**
	 * Create component container and place it on the dom
	 */
	helper.prepare = function( d_container ) {
		var node = domConstruct.toDom('<div class="networkdiagram" style="background-color:%0"></div>'.tokenize(backgroundColor));
        domConstruct.place(node, d_container);
        
        return node;
	};
	
	/**
	 * have the node link's container resize itself 
	 */
	helper.refresh = function( graph, graphContainer ) {
		_refresh(graph, graphContainer);
	};
	
	helper.dimEntity = function( entityArrayIndex, dim ) {
		this.networkProvider.rowStates.setHiddenAt(dim, entityArrayIndex);
	};

    helper.colorModels = {
        "borderColor": new DiscreteAttributesMap(),
        "linkColor": new DiscreteAttributesMap(),
        "linkStyle": new DiscreteAttributesMap()
    };

    /**
     * have to add this because we need to consume human color names from the data
     */
    helper.namedColorMap = {
        "black": "0x000000",
        "red": "0xCC0000",
        "orange": "0xFF6600",
        "yellow": "0xFFCC00",
        "green": "0x006600",
        "blue": "0x0000CC",
        "purple": "0x660066",
        "gray": "0x999999",
        "white": "0xFFFFFF"
    };

    /**
	 * setup and render component
	 */
	helper.start = function( controller, statefulEntityArray, graphContainer, callbackMap, callback ) {
        var graph = new Graph();
        
        var ac = statefulEntityArray;
        this.networkProvider = new ArrayCollectionDataProvider();
        this.networkProvider.setArrayCollection(ac);

        this.addToColorModel('borderColor', '0x000000');
        this.addToColorModel('linkColor', '0x808080');

        // default line styles
        this.addToColorModel('linkStyle', '0', {
            'lineStyle': 1
        });
        this.addToColorModel('linkStyle', '1', {
            'lineStyle': 1
        });
        this.addToColorModel('linkStyle', '2', {
            'lineStyle': 2
        });
        this.addToColorModel('linkStyle', '3', {
            'lineStyle': 3
        });
        this.addToColorModel('linkStyle', '4', {
            'lineStyle': 4
        });
        this.addToColorModel('linkStyle', '5', {
            'lineStyle': 5
        });


        ac.addEventListener(CollectionEvent.COLLECTION_CHANGE, function( event ) {
        	console.log(statefulEntityArray, 'statefulEntityArray');
        }, this);
        
		var riskModel = new ColorModel();
		riskModel.binned = false;
		riskModel._colorModelType = ColorModelTypeEnum.COLOR_LIST;
		riskModel.boundaryValuesNormalized = false;
		riskModel.colorList = [0xced7e0, 0x016cb2, 0x00FF00];

		var that = this;
	    _getTypeStyles( function( typeStyles ) {
            UserPreferenceUtil.retrieve('userpreferencewindow', function( showLabelsPreferenceResult ) {
                var scoreStyles = _getScoreStyles();

                // define networkDiagram
                var networkDiagram = new NetworkDiagram();
                networkDiagram.set("dataProvider", that.networkProvider);
                networkDiagram.set("nodeId", new Role('uid'));
                networkDiagram.set("linkId", new Role('uid'));
                networkDiagram.set("type", new Role('type'));
                networkDiagram.set("nodeX", new Role('positionX'));
                networkDiagram.set("nodeY", new Role('positionY'));
                networkDiagram.set("fromId", new Role('fromNodeId'));
                networkDiagram.set("toId", new Role('toNodeId'));
                networkDiagram.set("nodeIcon", new Role('nodeSymbol', null, null, typeStyles));

                networkDiagram.set("degree", new Role('nodeDegrees'));
                networkDiagram.set("nodeOutlineThicknessValue", new Role('borderSize'));

                networkDiagram.set("nodeOutlineColorGroup", new Role('borderColor', null, null, that.colorModels.borderColor));
                networkDiagram.set("nodeColorValue", new Role('nodeColor'));
                networkDiagram.set("highlightedNodes", new Role('highlight'));

                networkDiagram.set("groupId", new Role('uid'));
                networkDiagram.set("groupParentId", new Role('group'));
                networkDiagram.set("nodeParentId", new Role('group'));
                networkDiagram.set("groupIcon", new Role('nodeSymbol', null, null, typeStyles));
                networkDiagram.set("highlightedGroups", new Role('highlight'));

                //networkDiagram.set("groupOutlineColorGroup", new Role('score', null, null, scoreStyles));
                //networkDiagram.set("groupColorGroup",new Role('score', null, null, scoreStyles));

                var showNodeLabels = function() {
                    var show = true;

                    if( showLabelsPreferenceResult && showLabelsPreferenceResult.shownodelabelspreferenceselect ) {
                        if( showLabelsPreferenceResult.shownodelabelspreferenceselect.value === 'No' ) {
                            show = false;
                        }
                    }

                    return show;
                };
                // node labels
                if( showNodeLabels() === true ) {
                    networkDiagram.set("displayNodeLabels", true);
                    networkDiagram.set("nodeLabel", new Role('nodeLabel'));
                }
                else {
                    networkDiagram.set("displayNodeLabels", false);
                }

                networkDiagram.set("linkWidthMin", 2);
                networkDiagram.set("linkWidthMax", 10);
                // set link width property
                networkDiagram.set("linkWidthResponse", new Role('linkWidth'));
                networkDiagram.set("linkColorGroup", new Role('linkColor', null, null, that.colorModels.linkColor));
                networkDiagram.set("linkPatternGroup", new Role('linkStyle', null ,null, that.colorModels.linkStyle));
                networkDiagram.set("highlightedLinks", new Role('highlight'));
                networkDiagram.set("linkDirection", new Role('linkDirection'));

                var layoutRegion = new LayoutRegion();
                layoutRegion.set("region", networkDiagram);

                graph.backgroundAttributes.setStyle("BackFill", "0x" + backgroundColor);
                graph.backgroundAttributes.setStyle("transparency", 0);
                graph.set("layout", layoutRegion);
                graph.dataProvider = that.networkProvider;

                var graphView = (new GraphView(graphContainer)).placeAt(graphContainer);
                graphView.setGraphStyles(NLDThemeUtil.defaultTheme());
                graphView.set("graph", graph);
                graphView.setImageLibraryPath(IconUtil.displayListDefaultPath);
                this._graphView = graphView;

                var clicks = 0;
                var block = false;
                
                networkDiagram.addEventListener('mousedown', function (e) {
                	var map = controller.networkMapController.getMap();
			                    
			        if(_mapIsVisible(map)) {
                    	map.fire('focus');
			                    
                	//panning the map with the mouse
                    if (e.shiftKey === true) {
                        var evObj = document.createEvent('MouseEvents');
	                        evObj.initMouseEvent( 'mousedown', true, true, window, 1, e.screenX, e.screenY, e.clientX, e.clientY, false, false, false, false, 0, null );
	                        divmap.dispatchEvent(evObj);
		                    return;
	                    }
                    }
                }, that);
                
                networkDiagram.addEventListener('mouseup', function (e) {                    
                    clicks++;
                    setTimeout(function() {
                        if( clicks === 1 ) {
                            var selectedUids = that.getSelectedUids();

                            if( selectedUids.length === 1 ) {
                                callbackMap.entitySingleClick(selectedUids[0]);
                            }
                            else if( selectedUids.length > 1 ) {
                                callbackMap.multipleEntitiesSelected(selectedUids);
                            }
                        }
                        else {
                            if( block === false ) {
                                block = true;
                                var selectedUids = that.getSelectedUids();
                                if( selectedUids.length === 1 ) {
                                    callbackMap.entityDoubleClick(selectedUids[0]);
                                }
                                else if( selectedUids.length > 1 ) {
                                    callbackMap.multipleEntitiesSelected(selectedUids);
                                }
                            }
                            else {
                                block = false;
                            }
                        }
                        clicks = 0;
                    }, 300);

                    // traverse the picklists and grab the value of the passed-in field
                    var getSelectedEntity = function() {
                        var value;

                        var selectedUids = that.getSelectedUids();
                        if( selectedUids && selectedUids.length === 1 ) {
                            value = selectedUids[0];
                        }
                        return value;
                    };
                }, that);

                networkDiagram.addEventListener("mousewheel", function( e ) {
                    var map = controller.networkMapController.getMap();
                    var divmap = _mapIsVisible(map);

                    if(divmap) {
                        var evObj = document.createEvent('MouseEvents');
                        
                        //dealing with FF v. IE handling wheelDelta
                        if (e.wheelDelta == 120 || e.wheelDelta == 3) {
                        	var theWheelDelta = -3;
                        } else {
							var theWheelDelta = 3;                       
						}
                        
                        evObj.initMouseEvent('mousewheel', true, true, window, theWheelDelta, e.screenX, e.screenY, e.clientX, e.clientY, false, false, false, false, 0, null );
                        divmap.dispatchEvent(evObj);
                    }
                }, that);

                networkDiagram.addEventListener(NodeLinkDiagramActionEvent.NODE_EXPAND, function( e ) {
                    //
                    if( e.items && e.items.length > 0 ) {
                        var nodeItem = e.items[0];

                        var isGroup = (nodeItem.getType() === 'GROUP');
                        if( isGroup === false ) {
                            var map = controller.networkMapController.getMap();
                            var divmap = _mapIsVisible(map);

                            if (divmap) {//if the map is there
                                // don't expand the network if we are in the map view
                                map.fire('focus');
                                console.log("Don't expand network while in map view");
                            }
                            else {
                                var nodeType;
                                if( nodeItem ) {
                                    var acItem = ac.getItemAt(nodeItem.obsIndex);
                                    acItem.set('nodeDegrees', 0);
                                    nodeType = acItem.get('nodeType'); // doesn't come back as part of nodeItem?
                                }

                                controller.startGrowNetworkActions(nodeItem.get('uid'), nodeType);
                            }
                        }
                        else {
                            controller.removeGroup(nodeItem.get('uid'));
                        }
                    }
                }, that);

                networkDiagram.addEventListener("controllerOperationsAvailable", function (e) {
                    that.controllerOperations = e.controllerOperations;
                }, that);

                // resize graph when window is resized
                on(window, 'resize', function() {
                    _refresh(graph, graphContainer);
                });

                if( callback ) {
                    callback(graph);
                }
            });
	    });
        this.graph = graph;
	}; //  end of start

    helper.getSelectedUids = function () {
        var uids = [];
        var rowStates = this.networkProvider.rowStates;
        if (rowStates.getAnySelected()) {
            var selectedIndices = rowStates.selectedIndices;
            for (var i = rowStates.nSelectedIndices - 1; i >= 0; i--) {
                var obs = this.networkProvider._arrayCollection.getItemAt(selectedIndices[i]);
                uids.push(obs.uid);
            }
        }
        return uids;
    };

    helper.setDraggable = function (flag) {
        if(this.controllerOperations) {
            this.controllerOperations.editable = flag;
        }
    };

    helper.zoomIn = function() {
        this.controllerOperations.zoomIn();
    };

    helper.zoomOut = function() {
        this.controllerOperations.zoomOut();
    };

    helper.getZoom = function() {
        return this.controllerOperations.getZoomScale();
    };

    helper.setZoom = function(value) {
        this.controllerOperations.setZoomScale(value);
    };

	helper.panLeft = function() {
		var origLeft = this.controllerOperations.getHorizontalScrollPosition(); 
		this.controllerOperations.setHorizontalScrollPosition(origLeft + moveIncrement);
	};
	
	helper.panRight = function() {
		var origRight = this.controllerOperations.getHorizontalScrollPosition();
		this.controllerOperations.setHorizontalScrollPosition(origRight - moveIncrement);
	};

    helper.setTransparency = function(value) {
        this.graph.backgroundAttributes.setStyle("transparency", value);
    };

    helper.getTransparency = function() {
        this.graph.backgroundAttributes.getStyle("transparency");
    };

	helper.panUp = function() {
		var origUp = this.controllerOperations.getVerticalScrollPosition();
		this.controllerOperations.setVerticalScrollPosition(origUp - moveIncrement);
	};
	
	helper.panDown = function() {
		var origDown = this.controllerOperations.getVerticalScrollPosition();
		this.controllerOperations.setVerticalScrollPosition(origDown + moveIncrement);
	};
	
	helper.center = function() {
		this.controllerOperations.setViewportAlignment(HorizontalAlignmentEnum.CENTER, VerticalAlignmentEnum.CENTER, ViewportFitPolicyEnum.BOTH);
	};
	
	helper.localToData = function( point ) {
		return this.controllerOperations.localToData(point);
	};
	
	helper.displayAsideText = function( text ) {
		var d_snaDiagramContentPane = registry.byId('snadiagramcontentpane').domNode;
		var asideNode = domConstruct.toDom('<aside id="nldaside">%0</aside>'.tokenize(text));
		domConstruct.place(asideNode, d_snaDiagramContentPane, 'last');
	},

    helper.addToColorModel = function(model, colorString, attributes) {
        this.colorModels[model].existingColors = this.colorModels[model].existingColors || {};
        if(this.colorModels[model].existingColors[colorString]) {
            return; // already added
        }
        else {
            this.colorModels[model].existingColors[colorString] = true;
        }

        var style = new DiscreteAttributesMapEntry();
        style.values = [colorString];
        style.lineAttributes = new LineAttributes();

        if(attributes) {
            functional.forIn(attributes, function(value, key) {
                style.lineAttributes.setStyle(key, value);
            });
        }
        else {
            style.lineAttributes.setStyle("lineColor", colorString);
        }

        this.colorModels[model].entries.push(style);
    },

	helper.getCurrentlySelectedIndicies = function() {
		var selectedIndicies = [];
		for( var i = 0; i < this.networkProvider.rowStates.selected.length; i++ ) {
			var rowStateSelected = this.networkProvider.rowStates.selected[i];
			
			if( rowStateSelected === true ) {
				selectedIndicies.push(i);
			}
		}
		
		return selectedIndicies;
	};

    helper.getSelected = function () {
        var selected = [];
        var rowStates = this.networkProvider.rowStates;
        var ac = this.networkProvider._arrayCollection;
        if (rowStates.getAnySelected()) {
            var selectedIndices = rowStates.selectedIndices;
            for (var i = rowStates.nSelectedIndices - 1; i >= 0; i--) {
                selected.push(ac.getItemAt(selectedIndices[i]));
            }
        }

        return selected;
    };

    helper.createNewGroup = function (id, /*Array*/children) {
        //Can't create group with less than two items
        if (!children || children.length < 2) {
            return;
        }

        var groupUid = id;
        if( !groupUid ) {
            groupUid = GUIDUtil.generate();
        }

        var group = new Stateful({
            uid: groupUid,
            type: NodeLinkDiagramItemTypeEnum.GROUP,
            score: "group",
            nodeSymbol: GroupsHelper.GROUPICON,
            children: []
        });
        var i;
        for( i = children.length - 1; i >= 0; i-- ) {
            var node = children[i];
            node.group = group.uid;
            group.children.push(node);
        }

        if( group ) {
            this.networkProvider._arrayCollection.addItem(group);
            this.networkProvider.rowStates.setSelectedAll(false);
        }

        return group;
    };

    helper.groupSelected = function() {
        helper.createNewGroup(null, helper.getSelected());
    };
    
    return helper;
});
